from typing import List, Dict, Union

import pandas as pd
from roxieapi.cadata.CableDatabase import CableDatabase

from magnumapi.geometry.GeometryBuilder import GeometryBuilder, SlottedGeometryBuilder
from magnumapi.geometry.Geometry import Geometry
import pymbse.commons.json_file as json_file
from roxieapi import api


class GeometryFactory:
    """ GeometryFactory implements a factory design pattern and is used to produce:
    - rectangular geometry
    - absolute cos-theta geometry
    - relative cos-theta geometry
    - slotted absolute cos-theta geometry
    - slotted relative cos-theta geometry

    """

    @classmethod
    def init_geometry(cls, geometry_definition: Union[dict, str], cadata: CableDatabase) -> Geometry:
        """
        A class method initializing geometry from a geometry definition. The method supports the following inputs
        - if geometry_definition is a string path
            - if string is a path to a data file
                - a regular geometry is returned
            - if string is a path to a json file
                - read dictionary
                then:
                - either a slotted geometry is returned
                - or a regular geometry is returned
        """
        assert isinstance(geometry_definition, (str, dict)), \
            "Argument geometry_definition has to be either a str or dict"

        # if geometry_definition is a string and ends with .data
        # # return geometry initialized with data
        if isinstance(geometry_definition, str) and geometry_definition.endswith(".data"):
            return cls._init_with_data(geometry_definition, cadata)

        # if geometry is a string and ends with .json
        # # read json as a dict
        if isinstance(geometry_definition, str) and geometry_definition.endswith(".json"):
            geometry_definition = json_file.read(geometry_definition)

        # if the dictionary has keys block_defs, layer_defs, r_aperture
        # #  initialize a slotted geometry
        # else
        # # initialize regular geometry
        if len({"block_defs", "layer_defs", "r_aperture"}.intersection(geometry_definition)) == 3:
            return cls._init_slotted_with_dict(**geometry_definition, cadata=cadata)
        elif len({"block_defs", "layer_defs"}.intersection(geometry_definition)) == 2:
            return cls._init_with_dict(**geometry_definition, cadata=cadata)
        else:
            raise AttributeError("Unsupported combination of input keys in geometry_definition.")

    @classmethod
    def _init_with_dict(cls,
                       block_defs: List[Dict],
                       layer_defs: List[Dict],
                       cadata: CableDatabase) -> Geometry:
        """ Class method initializing a Geometry instance from a list of dictionaries with block definition.

        :param block_defs: a list of dictionaries with geometry definition (block definition)
        :param layer_defs: a list of dictionaries with layer definitions
        :param cadata: a CableDatabase instance
        :return: initialized geometry instance
        """
        return GeometryBuilder() \
            .with_block_defs(block_defs, cadata) \
            .with_layer_defs(layer_defs) \
            .build()

    @classmethod
    def _init_slotted_with_dict(cls,
                               block_defs: List[Dict],
                               layer_defs: List[Dict],
                               cadata: CableDatabase,
                               r_aperture: float) -> Geometry:
        """ Class method initializing a Geometry instance from a list of dictionaries with block definition.

        :param block_defs: a list of dictionaries with geometry definition (block definition)
        :param layer_defs: a list of dictionaries with layer definitions
        :param cadata: a CableDatabase instance
        :param r_aperture: aperture radius in mm
        :return: initialized geometry instance
        """
        return SlottedGeometryBuilder() \
            .with_block_defs(block_defs, cadata) \
            .with_layer_defs(layer_defs) \
            .with_r_aperture(r_aperture) \
            .build()

    @classmethod
    def _init_with_data(cls, data_file_path: str, cadata: CableDatabase) -> Geometry:
        """ Class method initializing a Geometry instance from a DATA ROXIE file.

        :param data_file_path: a path to a json file
        :param cadata: a CableDatabase instance
        :return: initialized geometry instance
        """
        block_df = api.read_bottom_header_table(data_file_path, keyword='BLOCK')
        layer_df = api.read_nested_bottom_header_table(data_file_path, keyword='LAYER')
        return cls._init_with_df(block_df, layer_df, cadata)

    @classmethod
    def _init_with_df(cls, block_df: pd.DataFrame, layer_df, cadata: CableDatabase) -> Geometry:
        """ Class method initializing a Geometry instance from a dataframe with block definition.

        :param block_df: a dataframe with geometry definition (block definition)
        :param cadata: a CableDatabase instance
        :return: initialized geometry instance
        """
        return GeometryBuilder() \
            .with_block_df(block_df, cadata) \
            .with_layer_df(layer_df) \
            .build()



from dataclasses import dataclass

from magnumapi.geometry.definitions.RectangularBlockDefinition import RectangularBlockDefinition


@dataclass
class HomogenizedRectangularBlockDefinition(RectangularBlockDefinition):
    """Class for a homogenized rectangular block definition.

    Attributes:
       radius_0 (float):
       radius_1 (float):
       radius_2 (float):
       radius_3 (float):
       phi_0 (float):
       phi_1 (float):
       phi_2 (float):
       phi_3 (float):
    """
    radius_0: float
    radius_1: float
    radius_2: float
    radius_3: float
    phi_0: float
    phi_1: float
    phi_2: float
    phi_3: float



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['magnumapi',
 'magnumapi.geometry',
 'magnumapi.geometry.blocks',
 'magnumapi.geometry.definitions',
 'magnumapi.geometry.primitives']

package_data = \
{'': ['*']}

install_requires = \
['Shapely>=1.8.2,<2.0.0',
 'ipyaggrid>=0.2.2,<0.3.0',
 'matplotlib>=3.5.2,<4.0.0',
 'numpy>=1.23.1,<2.0.0',
 'pandas>=1.4.3,<2.0.0',
 'pydantic>=1.9.1,<2.0.0',
 'pymbse-commons>=0.0.6,<0.0.7',
 'roxie-api>=0.0.12,<0.0.13']

setup_kwargs = {
    'name': 'magnum-api',
    'version': '0.5.4',
    'description': '',
    'long_description': '# Project Overview\nThe CHART MagNum (Magnum Numerics) aims at introducing advanced design techniques in order to support the design process of superconducting accelerator magnet.\nThe project responds to the following strategic goals:\n- Sustainability\n\n  Ensure that outstanding modeling work will have an impact on present or future designs. This also require that models have clearly defined scope and range of applicability.\n- Traceability\n\n  Ensure that the modeler is able to trace back the input parameters, code and script versions, etc., that have been used to produce a particular plot in a ppt or pdf.\n  Traceability is even more important in multi-scale and multi-model analysis.\n- Repeatability\n\n  Ensure that results presented at, e.g., a Conceptual Design Review can be reproduced at any later time.\n  Ensure that as-built models can be re-run at any moment during a potentially decades-long project life cycle.\n- Flexibility\n\n  Allow for different labs and collaborators to have/prefer different licenses.\n  Enable researchers to implement innovative ideas while building upon existing best practices, but without having to solve legacy issues.\n- Usability\n\n  Encapsulate the increased flexibility behind easy-to-use UIs for the standard design work.\n\nThe project implements a number of concepts introduced by the MBSE (Model-Based System Engineering) methdology.\nIn particular, MBSE shifts the focus from documents to models as primary means of communication in complex system design projects.\n',
    'author': 'mmaciejewski',
    'author_email': 'michal.maciejewski@ief.ee.ethz.ch',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://gitlab.cern.ch/chart-magnum/magnum-api',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<3.12',
}


setup(**setup_kwargs)
